﻿using System;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Runtime.Remoting.Metadata.W3cXsd2001;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace App
{
    public partial class Form1 : Form
    {
        private SerialHelper serialHelper = new SerialHelper();
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            tspNowTime.Text = "当前时间 : " + DateTime.Now.ToString("yyyy-MM-dd  hh:mm:ss");
            LocationClass.UploadRecord("打开软件");
            //加载串口列表
            cbxPort.Items.Clear();
            cbxPort.Items.AddRange(SerialPort.GetPortNames());
            if (cbxPort.Items.Count != 0)
            {
                cbxPort.SelectedIndex = 0;
                //指定默认波特率
                cbxBound.SelectedIndex = 1;
                InitSerialPort();
            }
            //IAP升级
            richbox = new Library.Richbox(tbx_IAP_Msg);
        }

        //初始化串口
        private void InitSerialPort()
        {
            try
            {
                serialPort1.Close();
                serialPort1.PortName = cbxPort.SelectedItem.ToString();
                if (cbxBound.SelectedItem==null)
                {
                    serialPort1.BaudRate = 0;
                }
                else
                { 
                    serialPort1.BaudRate = int.Parse(cbxBound.SelectedItem.ToString()); 
                }
                serialPort1.Open();
            }
            catch
            {

            }
        }


        private void cbxBound_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen)
                {
                    //关闭串口
                    serialPort1.Close();
                    InitSerialPort();
                    serialPort1.Open();
                }
            }
            catch
            {
            }
        }
        //选择串口是自动重新加载
        private void cbxPort_Click(object sender, EventArgs e)
        {
            int index = cbxPort.SelectedIndex;
            //加载串口列表
            cbxPort.Items.Clear();
            cbxPort.Items.AddRange(SerialPort.GetPortNames());
            if (cbxPort.Items.Count > 0)
            {
                if (index >= cbxPort.Items.Count)
                {
                    cbxPort.SelectedIndex = 0;
                }
                else
                    cbxPort.SelectedIndex = index;
            }
        }

        private void btnSerialPortOpenClose_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.Button btn = sender as System.Windows.Forms.Button;
            try
            {
                if (btn.Text == "关闭串口")
                {
                    serialPort1.Close();
                    btn.Text = "打开串口";
                    tspSerialPortState.Text = "已断开";
                    tspSerialPortState.ForeColor = Color.Red;
                    btn.ForeColor = Color.Black;
                }
                else
                {
                    InitSerialPort();
                    serialPort1.Close();
                    serialPort1.Open();
                    btn.Text = "关闭串口";
                    tspSerialPortState.Text = "已连接";
                    tspSerialPortState.ForeColor = Color.Green;
                    btn.ForeColor = Color.Red;
                }
            }
            catch
            {
                if (btn.Text == "打开串口")
                    MessageBox.Show("打开串口失败！");
            }

        }

        //实时显示系统时间
        private void timer1_Tick(object sender, EventArgs e)
        {
            tspNowTime.Text = "当前时间 : " + DateTime.Now.ToString("yyyy-MM-dd  hh:mm:ss");
        }
        //串口中断
        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                //等待数据接收完成
                if (serialPort1.BaudRate == 115200)
                    Thread.Sleep(10);
                else
                if (serialPort1.BaudRate == 9600)
                    Thread.Sleep(20);

                if (serialPort1.IsOpen == false)
                    return;
                if (systemRun == false)
                    return;
                // 开辟接收缓冲区
                byte[] bytes = new byte[serialPort1.BytesToRead];
                //从串口读取数据
                serialPort1.Read(bytes, 0, bytes.Length);
                serialHelper.reciveCount += bytes.Length;
                tspRecCount.Text = serialHelper.reciveCount.ToString();

                string str = Encoding.GetEncoding("gb2312").GetString(bytes);
                this.Invoke(new MethodInvoker(delegate
                {
                    //刷固件

                    if (bytes.Length < 8)
                        return;
                    //检验头
                    if (bytes[0] == 0xF1)
                    {
                        //检验crc
                        byte[] bt = CRC.CRC32_byte(bytes, bytes.Length - 4);
                        if (bytes[bytes.Length - 4] == bt[0] &&
                            bytes[bytes.Length - 3] == bt[1] &&
                            bytes[bytes.Length - 2] == bt[2] &&
                            bytes[bytes.Length - 1] == bt[3])
                        {
                            for (int i = 0; i < bytes.Length; i++)
                            {
                                ResponseData[i] = bytes[i];
                            }
                        }
                    }
                }));
            }
            catch
            {

            }
        }

        //点击网站连接
        private void toolStripStatusLabel2_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://igeekiot.com");
        }

        #region IAP[升级固件]
        static int PackageSize = 1024;
        byte[] databuf;
        byte[] datatosend = new byte[PackageSize];                      //定义数据发送缓冲区
        int PackageNum = 0;                                      //总包数
        int packet_zheng = 0;                                    //整包数
        int packet_yu = 0;                                       //不足1包的剩余字节数
        /// <summary>
        /// 已经发送整K字节个数
        /// </summary>
        public Library.Richbox richbox;                              //固件下载消息框
        Library.CRC CRC = new Library.CRC();
        //正在刷固件
        bool Running = false;
        byte[] ResponseData = new byte[1000];//请求响应

        int StartTimeOut = 20;//握手信号超时时间 ms
        int StartRepeat = 300;//握手信号重发次数 
        int timeout = 1000;//刷固件超时时间 ms
        /// <summary>
        /// 发送整包数据
        /// </summary>
        private void send_data_zheng(int index)
        {
            //单次发送的总长度
            int data_len = 4 + PackageSize + 4;
            byte[] Data = new byte[data_len];
            Array.Copy(databuf, index * PackageSize, datatosend, 0, PackageSize);
            //桢头
            Data[0] = 0xF1;
            //功能码
            Data[1] = 0x02;
            //包索引
            Data[2] = (byte)(index >> 8);
            Data[3] = (byte)index;
            for (int i = 0; i < PackageSize; i++)
            {
                Data[4 + i] = datatosend[i];
            }
            //CRC
            byte[] crc = CRC.CRC32_byte(Data, data_len - 4);
            //追加CRC
            Data[data_len - 4] = crc[0];
            Data[data_len - 3] = crc[1];
            Data[data_len - 2] = crc[2];
            Data[data_len - 1] = crc[3];
            Send(Data);
            //string str = string.Empty;
            //for (int j = 0; j < Data.Length; j++)
            //{
            //    str += String.Format("{0:X2}", Data[j]) + " ";
            //}
            //this.Invoke(new MethodInvoker(delegate
            //{
            //    richbox.Msg(str);
            //}));
        }
        /// <summary>
        /// 发送不足1包部分
        /// </summary>
        private void send_data_yu()
        {
            //单次发送的总长度
            int data_len = 4 + packet_yu + 4;
            byte[] Data = new byte[data_len];
            Array.Copy(databuf, packet_zheng * PackageSize, datatosend, 0, packet_yu);
            //桢头
            Data[0] = 0xF1;
            //功能码
            Data[1] = 0x02;
            //包索引
            Data[2] = (byte)(packet_zheng >> 8);
            Data[3] = (byte)packet_zheng;
            for (int i = 0; i < packet_yu; i++)
            {
                Data[4 + i] = datatosend[i];
            }
            //CRC
            byte[] crc = CRC.CRC32_byte(Data, data_len - 4);
            //追加CRC
            Data[data_len - 4] = crc[0];
            Data[data_len - 3] = crc[1];
            Data[data_len - 2] = crc[2];
            Data[data_len - 1] = crc[3];
            Send(Data);
            //string str = string.Empty;
            //for (int j = 0; j < Data.Length; j++)
            //{
            //    str += String.Format("{0:X2}", Data[j]) + " ";
            //}
            //this.Invoke(new MethodInvoker(delegate
            //{
            //    richbox.Msg(str);
            //}));
        }
        //iap发送函数
        public void Send(byte[] Data)
        {
            if (serialPort1.IsOpen == false)
                return;
            serialPort1.Write(Data, 0, Data.Length);
            serialHelper.sendCount += Data.Length;
            tspSendCount.Text = serialHelper.sendCount.ToString();
        }

        OpenFileDialog openfile = new OpenFileDialog();
        private void LoadFile()
        {
            string str = string.Empty;
            //判断文件是Bin还是Hex
            if (openfile.FileName.EndsWith(".bin"))
            {
                try
                {
                    //获取文件流
                    FileStream fs = new FileStream(openfile.FileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                    databuf = new byte[fs.Length];
                    fs.Read(databuf, 0, (int)fs.Length);
                    fs.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message.ToString());
                }

            }
            if (databuf != null)
            {
                packet_zheng = (int)databuf.Length / PackageSize;                                   //获取文件的整K字节数
                packet_yu = (int)databuf.Length % PackageSize;                                      //获取不足1K的剩余字节数
                PackageNum = packet_zheng + (packet_yu == 0 ? 0 : 1);
                this.Invoke(new MethodInvoker(delegate
                {
                    //显示文件的总字节数
                    richbox.Msg(Color.Blue, "文件共" + databuf.Length.ToString() + "字节");
                    //显示文件的总字节数
                    richbox.Msg(Color.DeepPink, "包大小：" + PackageSize.ToString());
                    //显示文件的总字节数
                    richbox.Msg(Color.DeepPink, "包总数：" + PackageNum.ToString());
                    //预计耗时
                    richbox.Msg(Color.DeepPink, "预计需要：" + (int)(databuf.Length / (115200 / 8.0) * 4) + "秒");
                }));
            }
        }
        private void btn_IAP_open_Click(object sender, EventArgs e)
        {
            try
            {
                openfile.Filter = @"固件(*.bin) | *.bin";
                //显示打开文件对话框
                openfile.ShowDialog();
                //获取所选择固件的名称
                txt_IAP_filename.Text = openfile.FileName;
                LoadFile();
                btn_IAP_send.Enabled = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        private void btn_IAP_send_Click(object sender, EventArgs e)
        {
            if (btn_IAP_send.Text == "下载")
            {
                if (serialPort1.IsOpen == false)
                {
                    try
                    {
                        serialPort1.Open();
                        tspSerialPortState.Text = "已连接";
                        button1.Text = "关闭串口";
                        tspSerialPortState.ForeColor = Color.Green;
                        button1.ForeColor = Color.Red;
                    }
                    catch
                    {
                        MessageBox.Show("串口未打开");
                        return;
                    }
                }
                else
                {
                    tspSerialPortState.Text = "已连接";
                    button1.Text = "关闭串口";
                    tspSerialPortState.ForeColor = Color.Green;
                    button1.ForeColor = Color.Red;
                }
                tbx_IAP_Msg.Text = string.Empty;
                LoadFile();
                Running = true;
                btn_IAP_send.Text = "取消";
                serialPort1.Close();
                if (serialPort1.BaudRate == 115200)
                {
                    serialPort1.BaudRate = 9600;
                }
                else
                {
                    serialPort1.BaudRate = 115200;
                }
                serialPort1.Open();
                progressBar1.Maximum = PackageNum;
                progressBar1.Value = 0;
                //启动刷固件进程
                Thread thread = new Thread(new ThreadStart(threadFun));
                thread.IsBackground = true;
                thread.Start();
            }
            else
            {
                Running = false;
                btn_IAP_send.Text = "下载";
                this.Invoke(new MethodInvoker(delegate
                {
                    richbox.Msg(string.Format("已取消！"));
                }));
                btn_IAP_open.Enabled = true;
            }
        }

        private void btn_IAP_ClrMsg_Click(object sender, EventArgs e)
        {
            tbx_IAP_Msg.Text = string.Empty;
        }

        //刷固件进程
        private void threadFun()
        {
            try
            {

                int timeCount = 0;
                Library.Time time = new Library.Time();
                long StartTime = time.NowTimeMS();
                while (Running)
                {
                    //0.请求刷固[用另一个波特率发]
                    for (int i = 0; i < 2; i++)
                    {
                        try
                        {
                            this.Invoke(new MethodInvoker(delegate
                            {
                                richbox.Msg(Color.Green, "{0} 第{1}次请求刷固...", DateTime.Now.ToString("HH:mm:ss fff"), i + 1);
                            }));
                        }
                        catch
                        {
                        }

                        byte[] Data = new byte[12];
                        //桢头
                        Data[0] = 0xF1;
                        //功能码
                        Data[1] = 0x01;
                        //固件长度
                        Data[2] = (byte)(databuf.Length >> 24);
                        Data[3] = (byte)(databuf.Length >> 16);
                        Data[4] = (byte)(databuf.Length >> 8);
                        Data[5] = (byte)(databuf.Length & 0xff);
                        //分包数
                        Data[6] = (byte)(PackageNum >> 8);
                        Data[7] = (byte)PackageNum;
                        //CRC
                        byte[] crc = CRC.CRC32_byte(Data, Data.Length - 4);
                        //追加CRC
                        Data[8] = crc[0];
                        Data[9] = crc[1];
                        Data[10] = crc[2];
                        Data[11] = crc[3];
                        Send(Data);


                        string str = string.Empty;
                        for (int j = 0; j < Data.Length; j++)
                        {
                            str += String.Format("{0:X2}", Data[j]) + " ";
                        }
                        this.Invoke(new MethodInvoker(delegate
                        {
                            richbox.Msg(str);
                        }));
                        Thread.Sleep(20);
                    }

                    //更换波特率
                    this.Invoke(new MethodInvoker(delegate
                    {
                        serialPort1.Close();
                        if (serialPort1.BaudRate == 9600)
                        {
                            serialPort1.BaudRate = 115200;
                        }
                        else
                        {
                            serialPort1.BaudRate = 9600;
                        }
                        serialPort1.Open();
                    }));



                    //1.请求刷固
                    for (int i = 0; i < StartRepeat; i++)
                    {
                        try
                        {
                            this.Invoke(new MethodInvoker(delegate
                            {
                                richbox.Msg(Color.Green, "{0} 第{1}次请求刷固...", DateTime.Now.ToString("HH:mm:ss fff"), i + 1);
                            }));
                        }
                        catch
                        {
                        }

                        byte[] Data = new byte[12];
                        //桢头
                        Data[0] = 0xF1;
                        //功能码
                        Data[1] = 0x01;
                        //固件长度
                        Data[2] = (byte)(databuf.Length >> 24);
                        Data[3] = (byte)(databuf.Length >> 16);
                        Data[4] = (byte)(databuf.Length >> 8);
                        Data[5] = (byte)(databuf.Length & 0xff);
                        //分包数
                        Data[6] = (byte)(PackageNum >> 8);
                        Data[7] = (byte)PackageNum;
                        //CRC
                        byte[] crc = CRC.CRC32_byte(Data, Data.Length - 4);
                        //追加CRC
                        Data[8] = crc[0];
                        Data[9] = crc[1];
                        Data[10] = crc[2];
                        Data[11] = crc[3];
                        Send(Data);
                        //Thread.Sleep(100);

                        string str = string.Empty;
                        for (int j = 0; j < Data.Length; j++)
                        {
                            str += String.Format("{0:X2}", Data[j]) + " ";
                        }
                        this.Invoke(new MethodInvoker(delegate
                        {
                            richbox.Msg(str);
                        }));

                        //等待响应
                        while (Running)
                        {
                            if (timeCount * 10 > StartTimeOut)
                            {
                                if (i == StartRepeat - 1)
                                {
                                    this.Invoke(new MethodInvoker(delegate
                                    {
                                        richbox.Msg(Color.Red, "{0} 等待超时，设备无响应，任务已取消！", DateTime.Now.ToString("HH:mm:ss fff"));
                                        Running = false;
                                        btn_IAP_send.Text = "下载";
                                    }));
                                    goto stop;
                                }
                                else
                                    break;
                            }
                            //收到响应，开始刷固件
                            if (ResponseData[1] == 0x81)
                            {
                                //清除接收缓存
                                Array.Clear(ResponseData, 0, ResponseData.Length);
                                goto AA;
                            }
                            Thread.Sleep(10);
                            timeCount++;
                        }
                        if (Running == false)
                        {
                            goto stop;
                        }
                        timeCount = 0;
                    }
                AA:
                    //更新启动时间
                    StartTime = time.NowTimeMS();
                    //2.开始刷固
                    //发送整包数据
                    for (int i = 0; i < packet_zheng; i++)
                    {
                        if (Running == false)
                            break;
                        this.Invoke(new MethodInvoker(delegate
                        {
                            progressBar1.Value = i + 1;
                            richbox.Msg(Color.Blue, "{0} 已发送第{1}包,共{2}包", DateTime.Now.ToString("HH:mm:ss fff"), i + 1, PackageNum);
                        }));
                        send_data_zheng(i);
                        //等待响应
                        timeCount = 0;
                        bool Sendflag = true;
                        while (Sendflag && Running)
                        {
                            //超时重发
                            if (timeCount * 10 > timeout)
                            {
                                this.Invoke(new MethodInvoker(delegate
                                {
                                    richbox.Msg(Color.Red, "{0} 等待超时，重发！", DateTime.Now.ToString("HH:mm:ss fff"));
                                    timeCount = 0;
                                    send_data_zheng(i);
                                    //Running = false;
                                    //btn_IAP_send.Text = "下载";
                                }));
                                //goto stop;
                            }
                            //收到了反馈
                            if (ResponseData[1] == 0x82)
                            {
                                int idx = (ResponseData[2] << 8) | ResponseData[3];
                                //包索引是否一致
                                if (i == idx)
                                {
                                    //判断结果
                                    switch (ResponseData[4])
                                    {
                                        case 0x01:
                                            //正常
                                            if ((idx + 1) == PackageNum)
                                            {
                                                //升级完成
                                                this.Invoke(new MethodInvoker(delegate
                                                {
                                                    richbox.Msg(Color.Green, "{0} Flash写入成功！", DateTime.Now.ToString("HH:mm:ss fff"));
                                                    richbox.Msg(Color.Red, "完成！");
                                                }));
                                                Running = false;
                                                btn_IAP_send.Text = "下载";
                                                goto stop;
                                            }
                                            else
                                            {
                                                //退出循环，继续下一包发送
                                                Sendflag = false;
                                            }
                                            break;
                                        case 0x02:
                                        case 0x03:
                                            //数据错误，重新发送
                                            this.Invoke(new MethodInvoker(delegate
                                            {
                                                richbox.Msg(Color.Red, "{0} 等待超时，重发！", DateTime.Now.ToString("HH:mm:ss fff"));
                                            }));
                                            timeCount = 0;
                                            send_data_zheng(i);
                                            break;
                                        default:
                                            break;
                                    }
                                }
                                //清除接收缓存
                                Array.Clear(ResponseData, 0, ResponseData.Length);
                            }
                            Thread.Sleep(10);
                            timeCount++;
                        }
                    }

                    //发送剩余数据
                    if (Running)
                    {
                        send_data_yu();
                        this.Invoke(new MethodInvoker(delegate
                        {
                            richbox.Msg(Color.Blue, "{0} 已发送第{1}包,共{2}包", DateTime.Now.ToString("HH:mm:ss fff"), PackageNum, PackageNum);
                        }));
                    }
                    //等待响应
                    timeCount = 0;
                    while (Running)
                    {
                        //超时重发
                        if (timeCount * 10 > timeout)
                        {
                            this.Invoke(new MethodInvoker(delegate
                            {
                                richbox.Msg(Color.Red, "{0} 等待超时，重发！", DateTime.Now.ToString("HH:mm:ss fff"));
                                timeCount = 0;
                                send_data_yu();
                            }));
                        }
                        //收到了反馈
                        if (ResponseData[1] == 0x82)
                        {
                            int idx = (ResponseData[2] << 8) | ResponseData[3];
                            //包索引是否一致
                            if ((PackageNum - 1) == idx)
                            {
                                //判断结果
                                switch (ResponseData[4])
                                {
                                    case 0x01:
                                        //升级完成
                                        this.Invoke(new MethodInvoker(delegate
                                        {
                                            progressBar1.Value = progressBar1.Maximum;
                                            richbox.Msg(Color.Green, "{0} Flash写入成功！", DateTime.Now.ToString("HH:mm:ss fff"));
                                            richbox.Msg(Color.Green, "完成！");
                                            Running = false;
                                            btn_IAP_send.Text = "下载";
                                        }));
                                        goto stop;
                                    // break;
                                    case 0x02:
                                    case 0x03:
                                        //数据错误，重新发送
                                        this.Invoke(new MethodInvoker(delegate
                                        {
                                            richbox.Msg(Color.Red, "{0} 等待超时，重发！", DateTime.Now.ToString("HH:mm:ss fff"));
                                        }));
                                        timeCount = 0;
                                        send_data_yu();
                                        break;
                                    default:
                                        break;
                                }
                            }
                            //清除接收缓存
                            Array.Clear(ResponseData, 0, ResponseData.Length);
                        }
                        Thread.Sleep(10);
                        timeCount++;
                    }
                }
            stop:;
                this.Invoke(new MethodInvoker(delegate
                {
                    //获取结束时间
                    long StopTime = (DateTime.Now.ToUniversalTime().Ticks - 621355968000000000) / 10000;
                    richbox.Msg(Color.Green, "耗时：{0}ms", StopTime - StartTime);
                }));
                Thread.Sleep(1100);
            }
            catch
            {
            }
            //下载完成后自动关闭串口
            this.Invoke(new MethodInvoker(delegate
            {
                serialPort1.Close();
                button1.Text = "打开串口";
                tspSerialPortState.Text = "已断开";
                tspSerialPortState.ForeColor = Color.Red;
                button1.ForeColor = Color.Black;
                richbox.Msg(Color.Blue, "提示：已自动关闭串口！");
            }));

        }
        #endregion



        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            LocationClass.UploadRecord("关闭软件");
        }
        bool systemRun = true;
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            systemRun = false;
            serialPort1.DataReceived -= serialPort1_DataReceived;
            Thread.Sleep(300);
        }

        private void cbxPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {

                //关闭串口
                serialPort1.Close();
                InitSerialPort();
                serialPort1.Open();

            }
            catch
            {
            }
        }

        private void cbxPort_Click_1(object sender, EventArgs e)
        {
            //加载串口列表
            cbxPort.Items.Clear();
            cbxPort.Items.AddRange(SerialPort.GetPortNames());
            if (cbxPort.Items.Count != 0)
            {
                cbxPort.SelectedIndex = 0;
                //指定默认波特率
                cbxBound.SelectedIndex = 1;
                InitSerialPort();
            }
        }
    }
}
